/* Copyright (c) 2009-2010 Ondrej Donek.
   See the file LICENSE.txt for licensing information. */

/**
 * Main controller object of note dialog
 * 
 * @type Object
 */
var tbNoteDialog = {
	
	get mStrbundle() { return document.getElementById("tbnotes-stringbundle"); },
	
	get DEFAULT_EDIT_MODE() { return "plain"; },
	get DEFAULT_EDIT_FONT_STYLE() { return ""; },
	get DEFAULT_EDIT_FONT_SIZE() { return "xx-small"; },
	
	/** 
	 * Holds instance of object describing target email message
	 * @type Object|NULL
	 */
	_targetMsg: null,
	
	/**
	 * Holds instance of object representing edited note.
	 * @type Object|NULL
	 */
	_editedNote: null,
	
	/**
	 * Fired when dialog is loading.
	 * 
	 * @return void
	 */
	onLoad: function() 
	{
		var op = "add";
		
		if(typeof(window.arguments[0].inn.note) != "undefined") {
			if(window.arguments[0].inn.note != null) {
				op = "edit";
			}
		}
		
		if(op == "add") {
			// Check if we targeting note to some message
			// TODO When we editing note target message can be blank so load subject by self here!
			if(typeof(window.arguments[0].inn.msg) != "undefined") {
				if(window.arguments[0].inn.msg != null) {
					this._targetMsg = window.arguments[0].inn.msg;
				}
			}
			this._noteLinks = new Array();
		}
		else {
			this._editedNote = window.arguments[0].inn.note;
			this._noteLinks = this._editedNote.getLinks();
		}
		
		this.updateDialog();
	},
	
	_editmode: "plain",
	_fontstyle: "",
	_fontsize: "xx-small",
	
	_updateStylePrefs: function() {
		try {
			var prefs = Components.classes["@mozilla.org/preferences-service;1"].
					getService(Components.interfaces.nsIPrefBranch);
			var pref1 = "extensions.tbnotes.note_dialog_editor_mode";
			var pref2 = "extensions.tbnotes.note_dialog_editor_font";
			var pref3 = "extensions.tbnotes.note_dialog_editor_font_size";
			
			this._editmode  = prefs.getCharPref(pref1);
			this._fontstyle = prefs.getCharPref(pref2);
			this._fontsize  = prefs.getCharPref(pref3);
		} catch(e) {
			this._editmode = this.DEFAULT_EDIT_MODE;
			this._fontstyle = this.DEFAULT_EDIT_FONT_STYLE;
			this._fontsize = this.DEFAULT_EDIT_FONT_SIZE;
		}
	},
	
	/** 
	 * Updates dialog's UI.
	 * 
	 * @return void
	 */
	updateDialog: function() 
	{
		var dlg = document.getElementById("tbnotes-note-dialog");
		var cont = document.getElementById("tbnotedialog_createdCont-hbox");
		var target = document.getElementById("tbnotedialog_target-textbox");
		var created = document.getElementById("tbnotedialog_created-textbox");
		var title = document.getElementById("tbnotedialog_title-textbox");
		var body = document.getElementById("tbnotedialog_body-editor");
		var editor = null;
		var toolbar = document.getElementById("FormatToolbar");
		
		this._updateStylePrefs();
		
		if(this._editmode == "plain") {
			// Load note template
			//body.contentWindow.location = tbNotesUtils.getNoteTemplateFileUri("text");
			// Initialize editor self
			editor = body.getEditor(body.contentWindow);
			body.makeEditable("text/plain", false);
			editor.QueryInterface(Components.interfaces.nsIPlaintextEditor);
			editor.enableUndo(false);
			editor.rootElement.style.fontFamily = this._fontstyle;
			editor.rootElement.style.fontSize = this._fontsize;
			editor.rootElement.style.whiteSpace = "pre";
			editor.rootElement.style.margin = 5;
			toolbar.collapsed = true;
		} else {
			// Load note template
			//body.contentWindow.location = tbNotesUtils.getNoteTemplateFileUri("html");
			// Initialize editor self
			editor = body.getHTMLEditor(body.contentWindow);
			body.makeEditable("text/xml", false);
			editor.QueryInterface(Components.interfaces.nsIHTMLEditor);
			editor.enableUndo(false);
			editor.rootElement.style.fontFamily = this._fontstyle;
			editor.rootElement.style.fontSize = this._fontsize;
			//editor.rootElement.style.whiteSpace = "pre";
			editor.rootElement.style.margin = 5;
			toolbar.collapsed = false;
		}
		
		if(this._editedNote != null) {
			// We are editing given note
			target.value = this._editedNote.mTargetName;
			target.setAttribute("tooltiptext", this._editedNote.mTargetName);
			created.value = this._editedNote.mCreated;
			title.value = this._editedNote.mTitle;
			
			(this._editmode == "plain")
					? editor.insertText(this._editedNote.mBody)
					: editor.insertHTML("<html><head/><body>" + this._editedNote.mBody + "</body></html");
			editor.resetModificationCount();
			editor.addDocumentStateListener({
				NotifyDocumentCreated: function() {},
				NotifyDocumentWillBeDestroyed: function() {}, 
				NotifyDocumentStateChanged: function(aIsChanged) {}
			});
			
			if(cont.hasAttribute("collapsed")) {
				cont.removeAttribute("collapsed");
			}
			
			// TODO Use stringbundles!
			var dlgTitle = dlg.getAttribute("titleEdit");
			if(typeof(this._editedNote.mId) != "undefined") {
				dlgTitle = dlgTitle + " #" + this._editedNote.mId;
			}
			dlg.setAttribute("title", dlgTitle);
		} else {
			// We are adding new note
			
			// Has note any target?
			if(this._targetMsg != null) {
				target.value = this._targetMsg.subject;
				target.setAttribute("tooltiptext", this._targetMsg.subject);
			} else {
				target.value = tbNoteDialog.mStrbundle.getString("noTargetLbl");
			}
			
			// Hide grid row with created textbox
			cont.setAttribute("collapsed", "true");
			dlg.setAttribute("title", dlg.getAttribute("titleAdd"));
		}
		
		this.updateLinks();
		
		// Select title textbox
		title.focus();
	},
	
	/**
	 * Update links UI
	 *
	 * @returns {void}
	 */
	updateLinks: function()
	{
		var XUL_NS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
		var links = document.getElementById("tbnotedialog_links-description");
		// Remove current links
		tbNotesUtils.removeChilds(links);
		
		if(this._noteLinks.length == 0) {
			// Append 'No links attached' labels
			var label = document.createElementNS(XUL_NS, "label");
			label.setAttribute("value", tbNoteDialog.mStrbundle.getString("noLinksAttached"));
			label.setAttribute("class", "noLinksAttached");
			links.appendChild(label);
		}
		else {
			// Display all links
			for(var i=0; i<this._noteLinks.length; i++) {
				var label = document.createElementNS(XUL_NS, "label");
				label.setAttribute("value", this._noteLinks[i].mTitle);
				label.setAttribute("tooltiptext", "URL: " + this._noteLinks[i].mUrl);
				label.setAttribute("class", "link editLinkCmd");
				//label.setAttribute("popup", "tbnotedialog_link-popup");
				label.setAttribute("onclick", "tbNoteDialog.onLinkClick(event, '" + i + "');");
				links.appendChild(label);
			}
		}
	},
	
	/** 
	 * Fired when user accepted the note dialog.
	 * 
	 * @return boolean
	 */
	onAccept: function()
	{
		try {
			var titleElm = document.getElementById("tbnotedialog_title-textbox");
			var body = document.getElementById("tbnotedialog_body-editor");
			var editor = body.getEditor(body.contentWindow);
			
			if(titleElm.value == "") {
				var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].
						getService(Components.interfaces.nsIPromptService);
				var title = tbLinkDialog.mStrbundle.getString("errorDlgTitle");
				var msg = tbNoteDialog.mStrbundle.getString("noteTitleWarn");
				
				prompts.alert(window, title, msg);
			}
			
			// TODO What if target was cleared by user?
			
			if(this._editedNote == null) {
				var noteObj = null;
				var win = this.getMessengerWin();
				with(win) {
					noteObj = new tbNotesNotePrototype();
				}
				
				// We are adding new note
				this._editedNote = noteObj;
				
				if(this._targetMsg != null) {
					var isNews = this._targetMsg.selectedMessageIsNews;
					var isFeed = this._targetMsg.selectedMessageIsFeed;
					
					if(isNews) {
						this._editedNote.mTargetType = "newsitem";
					} else if(isFeed) {
						this._editedNote.mTargetType = "feeditem";
					} else {
						this._editedNote.mTargetType = "mailmessage";
					}
					// TODO `address book items` => `abitem`
					
					this._editedNote.mTargetKey = this._targetMsg.key;
					this._editedNote.mTargetName = this._targetMsg.subject;
					this._editedNote.mTargetId = this._targetMsg.id;
				}
			}
			
			this._editedNote.mTitle = titleElm.value;
			this._editedNote.mBody = (this._editmode == "plain")
					? editor.outputToString("text/plain", 1024)
					: editor.outputToString("text/xml", 1024);
			
			this._editedNote.setLinks(this._noteLinks);
			
			window.arguments[0].out = this._editedNote;
		} catch(e) {
			Components.utils.reportError(e);
		}
		
		return true;
	}, 
	
	/**
	 * Fired when user canceled the note dialog.
	 * 
	 * @return boolean
	 */
	onCancel: function()
	{
		var ret = true;
		
		var win = this.getMessengerWin();
		with(win) {
			var prefs = Components.classes["@mozilla.org/preferences-service;1"].
					getService(Components.interfaces.nsIPrefBranch);
			var pref = "extensions.tbnotes.dont_show_close_confirm";
			
			// Check if user want to see confirmation dialog
			if(prefs.getBoolPref(pref) === true) {
				return true;
			}
			
			// Show confirmation if user really want to close unsaved note
			var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].
					getService(Components.interfaces.nsIPromptService);
			
			var title = tbNoteDialog.mStrbundle.getString("closeNoteDlgTitle");
			var desc = tbNoteDialog.mStrbundle.getString("closeNoteDlgDesc");
			var question = tbNoteDialog.mStrbundle.getString("doNotAskAgain");
			var check = { value: false };
			ret = prompts.confirmCheck(window, title, desc, question, check);
			
			prefs.setBoolPref(pref, check.value);
		}
		
		return res;
	},
	
	/**
	 * Performs requested command
	 * 
	 * @param aCmdName {string}
	 * @return void
	 */
	doCommand: function(aCmdName)
	{
		var win = this.getMessengerWin();
		
		switch(aCmdName) {
			// Add new link command
			case "add_link":
				with(win) {
					var params = { inn: null, out: null };
					window.openDialog("chrome://tbnotes/content/link-dialog.xul", "",
														"chrome,extrachrome,dialog,toolbar,modal",
														params).focus();
					
					if(params.out) {
						var link = new tbNotesLinkPrototype();
						link.mTitle = params.out.mTitle;
						link.mUrl = params.out.mUrl;
						
						this._noteLinks.push(link);
						this.updateLinks();
					}
				}
				break;
			
			// Edit link
			case "edit_link":
				with(win) {
					var link = this._noteLinks[this._link_index];
					
					if(link == null) return;
						
					var params = { inn: link, out: null };
					window.openDialog("chrome://tbnotes/content/link-dialog.xul", "",
														"chrome,extrachrome,dialog,toolbar,modal",
														params).focus();
					
					if(params.out) {
						this._noteLinks[this._link_index].mTitle = params.out.mTitle;
						this._noteLinks[this._link_index].mUrl = params.out.mUrl;
						this.updateLinks();
					}
				}
				break;
			
			// Remove selected link
			case "remove_link":
				with(win) {
					var link = this._noteLinks[this._link_index];
					
					if(link == null) return;
					
					var prefs = Components.classes["@mozilla.org/preferences-service;1"].
							getService(Components.interfaces.nsIPrefBranch);
					var pref = "extensions.tbnotes.dont_show_remove_confirm";
					
					// Check if user want to see confirmation dialog
					if(prefs.getBoolPref(pref) === true) {
						// Show confirmation if user really want to close unsaved note
						var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].
								getService(Components.interfaces.nsIPromptService);
						
						var title = tbNoteDialog.mStrbundle.getString("deleteLinkDlgTitle");
						var desc = tbNoteDialog.mStrbundle.getFormattedString("deleteLinkDlgTitle", [link.mTitle]);
						var question = tbNoteDialog.mStrbundle.getString("doNotAskAgain");
						var check = { value: false };
						var res = prompts.confirmCheck(window, title, desc, question, check);
						
						prefs.setBoolPref(pref, check.value);
						
						if(res) {
							link.remove();
							this._noteLinks.splice(this._link_index, 1);
							this.updateLinks();
						}
					} else {
						link.remove();
						this._noteLinks.splice(this._link_index, 1);
						this.updateLinks();
					}
				}
				break;
			
			// Show link in the browser
			case "show_link":
				with(win) {
					var link = this._noteLinks[this._link_index];
					
					if(link != null) {
						tbNotesUtils.openLinkInExternalBrowser(link.mUrl);
					}
				}
				break;
			
			// Copy URL of the selected link to the clipboard
			case "copy_link":
				with(win) {
					var link = this._noteLinks[this._link_index];
					
					if(link != null) {
						tbNotesUtils.copyTextToClipboard(link.mUrl);
					}
				}
				break;
			
			default:
				Components.utils.reportError("tbNoteDialog.doCommand:: undefined command '" + aCmdName + "'.");
				break;
		}
	},
	
	/**
	 * Fired when user clicked on any link. It shows the link's context menu.
	 *
	 * @param aEvent {Components.interfaces.nsIDOMEvent}
	 * @param aLinkIndex {integer} Index of link in tbNotesDialog._noteLinks
	 * @returns void
	 */
	onLinkClick: function(aEvent, aLinkIndex)
	{
		// We can not use link.mId here!
		this._link_index = aLinkIndex;
		
		var popup = document.getElementById("tbnotedialog_link-popup");
		popup.openPopup(aEvent.target, "after_pointer", null, null, true);
	},
	
	/**
	 * Fired when link's context menu is going to be hidden. It erases ID of the link
	 * from tbNotesDialog object.
	 *
	 * @returns void
	 */
	onLinkContextHiding: function()
	{
		delete this._link_index;
	},
	
	/**
	 * Returns object of the main Thunderbird's window
	 *
	 * @return {Components.interfaces.nsIXULWindow}
	 */
	getMessengerWin: function()
	{
		var winMan = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService();
		var winManInt = winMan.QueryInterface(Components.interfaces.nsIWindowMediator);
		
		return winManInt.getMostRecentWindow("mail:3pane");
	}
	
}; // End of tbNoteDialog
